#include "rust/cxx.h"

namespace rust {
inline namespace cxxbridge1 {
namespace detail {
template <typename T, typename = void *>
struct operator_new {
  void *operator()(::std::size_t sz) { return ::operator new(sz); }
};

template <typename T>
struct operator_new<T, decltype(T::operator new(sizeof(T)))> {
  void *operator()(::std::size_t sz) { return T::operator new(sz); }
};
} // namespace detail

template <typename T>
union MaybeUninit {
  T value;
  void *operator new(::std::size_t sz) { return detail::operator_new<T>{}(sz); }
  MaybeUninit() {}
  ~MaybeUninit() {}
};
} // namespace cxxbridge1
} // namespace rust

namespace orchard_bundle {
  struct Action;
  struct Bundle;
  struct OrchardBundle;
}

namespace orchard_bundle {
#ifndef CXXBRIDGE1_STRUCT_orchard_bundle$Action
#define CXXBRIDGE1_STRUCT_orchard_bundle$Action
struct Action final : public ::rust::Opaque {
  ::std::array<::std::uint8_t, 32> cv() const noexcept;
  ::std::array<::std::uint8_t, 32> nullifier() const noexcept;
  ::std::array<::std::uint8_t, 32> rk() const noexcept;
  ::std::array<::std::uint8_t, 32> cmx() const noexcept;
  ::std::array<::std::uint8_t, 32> ephemeral_key() const noexcept;
  ::std::array<::std::uint8_t, 580> enc_ciphertext() const noexcept;
  ::std::array<::std::uint8_t, 80> out_ciphertext() const noexcept;
  ::std::array<::std::uint8_t, 64> spend_auth_sig() const noexcept;
  ~Action() = delete;

private:
  friend ::rust::layout;
  struct layout {
    static ::std::size_t size() noexcept;
    static ::std::size_t align() noexcept;
  };
};
#endif // CXXBRIDGE1_STRUCT_orchard_bundle$Action

#ifndef CXXBRIDGE1_STRUCT_orchard_bundle$Bundle
#define CXXBRIDGE1_STRUCT_orchard_bundle$Bundle
struct Bundle final : public ::rust::Opaque {
  ::rust::Vec<::orchard_bundle::Action> actions() const noexcept;
  ::std::size_t num_actions() const noexcept;
  bool enable_spends() const noexcept;
  bool enable_outputs() const noexcept;
  ::std::int64_t value_balance_zat() const noexcept;
  ::std::array<::std::uint8_t, 32> anchor() const noexcept;
  ::rust::Vec<::std::uint8_t> proof() const noexcept;
  ::std::array<::std::uint8_t, 64> binding_sig() const noexcept;
  ~Bundle() = delete;

private:
  friend ::rust::layout;
  struct layout {
    static ::std::size_t size() noexcept;
    static ::std::size_t align() noexcept;
  };
};
#endif // CXXBRIDGE1_STRUCT_orchard_bundle$Bundle

#ifndef CXXBRIDGE1_STRUCT_orchard_bundle$OrchardBundle
#define CXXBRIDGE1_STRUCT_orchard_bundle$OrchardBundle
struct OrchardBundle final : public ::rust::Opaque {
  ~OrchardBundle() = delete;

private:
  friend ::rust::layout;
  struct layout {
    static ::std::size_t size() noexcept;
    static ::std::size_t align() noexcept;
  };
};
#endif // CXXBRIDGE1_STRUCT_orchard_bundle$OrchardBundle

extern "C" {
::std::size_t orchard_bundle$cxxbridge1$Action$operator$sizeof() noexcept;
::std::size_t orchard_bundle$cxxbridge1$Action$operator$alignof() noexcept;
::std::size_t orchard_bundle$cxxbridge1$Bundle$operator$sizeof() noexcept;
::std::size_t orchard_bundle$cxxbridge1$Bundle$operator$alignof() noexcept;
::std::size_t orchard_bundle$cxxbridge1$OrchardBundle$operator$sizeof() noexcept;
::std::size_t orchard_bundle$cxxbridge1$OrchardBundle$operator$alignof() noexcept;

void orchard_bundle$cxxbridge1$Action$cv(::orchard_bundle::Action const &self, ::std::array<::std::uint8_t, 32> *return$) noexcept;

void orchard_bundle$cxxbridge1$Action$nullifier(::orchard_bundle::Action const &self, ::std::array<::std::uint8_t, 32> *return$) noexcept;

void orchard_bundle$cxxbridge1$Action$rk(::orchard_bundle::Action const &self, ::std::array<::std::uint8_t, 32> *return$) noexcept;

void orchard_bundle$cxxbridge1$Action$cmx(::orchard_bundle::Action const &self, ::std::array<::std::uint8_t, 32> *return$) noexcept;

void orchard_bundle$cxxbridge1$Action$ephemeral_key(::orchard_bundle::Action const &self, ::std::array<::std::uint8_t, 32> *return$) noexcept;

void orchard_bundle$cxxbridge1$Action$enc_ciphertext(::orchard_bundle::Action const &self, ::std::array<::std::uint8_t, 580> *return$) noexcept;

void orchard_bundle$cxxbridge1$Action$out_ciphertext(::orchard_bundle::Action const &self, ::std::array<::std::uint8_t, 80> *return$) noexcept;

void orchard_bundle$cxxbridge1$Action$spend_auth_sig(::orchard_bundle::Action const &self, ::std::array<::std::uint8_t, 64> *return$) noexcept;

::orchard_bundle::Bundle *orchard_bundle$cxxbridge1$from_tx_bundle(::orchard_bundle::OrchardBundle const *bundle) noexcept;

void orchard_bundle$cxxbridge1$Bundle$actions(::orchard_bundle::Bundle const &self, ::rust::Vec<::orchard_bundle::Action> *return$) noexcept;

::std::size_t orchard_bundle$cxxbridge1$Bundle$num_actions(::orchard_bundle::Bundle const &self) noexcept;

bool orchard_bundle$cxxbridge1$Bundle$enable_spends(::orchard_bundle::Bundle const &self) noexcept;

bool orchard_bundle$cxxbridge1$Bundle$enable_outputs(::orchard_bundle::Bundle const &self) noexcept;

::std::int64_t orchard_bundle$cxxbridge1$Bundle$value_balance_zat(::orchard_bundle::Bundle const &self) noexcept;

void orchard_bundle$cxxbridge1$Bundle$anchor(::orchard_bundle::Bundle const &self, ::std::array<::std::uint8_t, 32> *return$) noexcept;

void orchard_bundle$cxxbridge1$Bundle$proof(::orchard_bundle::Bundle const &self, ::rust::Vec<::std::uint8_t> *return$) noexcept;

void orchard_bundle$cxxbridge1$Bundle$binding_sig(::orchard_bundle::Bundle const &self, ::std::array<::std::uint8_t, 64> *return$) noexcept;
} // extern "C"

::std::size_t Action::layout::size() noexcept {
  return orchard_bundle$cxxbridge1$Action$operator$sizeof();
}

::std::size_t Action::layout::align() noexcept {
  return orchard_bundle$cxxbridge1$Action$operator$alignof();
}

::std::size_t Bundle::layout::size() noexcept {
  return orchard_bundle$cxxbridge1$Bundle$operator$sizeof();
}

::std::size_t Bundle::layout::align() noexcept {
  return orchard_bundle$cxxbridge1$Bundle$operator$alignof();
}

::std::size_t OrchardBundle::layout::size() noexcept {
  return orchard_bundle$cxxbridge1$OrchardBundle$operator$sizeof();
}

::std::size_t OrchardBundle::layout::align() noexcept {
  return orchard_bundle$cxxbridge1$OrchardBundle$operator$alignof();
}

::std::array<::std::uint8_t, 32> Action::cv() const noexcept {
  ::rust::MaybeUninit<::std::array<::std::uint8_t, 32>> return$;
  orchard_bundle$cxxbridge1$Action$cv(*this, &return$.value);
  return ::std::move(return$.value);
}

::std::array<::std::uint8_t, 32> Action::nullifier() const noexcept {
  ::rust::MaybeUninit<::std::array<::std::uint8_t, 32>> return$;
  orchard_bundle$cxxbridge1$Action$nullifier(*this, &return$.value);
  return ::std::move(return$.value);
}

::std::array<::std::uint8_t, 32> Action::rk() const noexcept {
  ::rust::MaybeUninit<::std::array<::std::uint8_t, 32>> return$;
  orchard_bundle$cxxbridge1$Action$rk(*this, &return$.value);
  return ::std::move(return$.value);
}

::std::array<::std::uint8_t, 32> Action::cmx() const noexcept {
  ::rust::MaybeUninit<::std::array<::std::uint8_t, 32>> return$;
  orchard_bundle$cxxbridge1$Action$cmx(*this, &return$.value);
  return ::std::move(return$.value);
}

::std::array<::std::uint8_t, 32> Action::ephemeral_key() const noexcept {
  ::rust::MaybeUninit<::std::array<::std::uint8_t, 32>> return$;
  orchard_bundle$cxxbridge1$Action$ephemeral_key(*this, &return$.value);
  return ::std::move(return$.value);
}

::std::array<::std::uint8_t, 580> Action::enc_ciphertext() const noexcept {
  ::rust::MaybeUninit<::std::array<::std::uint8_t, 580>> return$;
  orchard_bundle$cxxbridge1$Action$enc_ciphertext(*this, &return$.value);
  return ::std::move(return$.value);
}

::std::array<::std::uint8_t, 80> Action::out_ciphertext() const noexcept {
  ::rust::MaybeUninit<::std::array<::std::uint8_t, 80>> return$;
  orchard_bundle$cxxbridge1$Action$out_ciphertext(*this, &return$.value);
  return ::std::move(return$.value);
}

::std::array<::std::uint8_t, 64> Action::spend_auth_sig() const noexcept {
  ::rust::MaybeUninit<::std::array<::std::uint8_t, 64>> return$;
  orchard_bundle$cxxbridge1$Action$spend_auth_sig(*this, &return$.value);
  return ::std::move(return$.value);
}

::rust::Box<::orchard_bundle::Bundle> from_tx_bundle(::orchard_bundle::OrchardBundle const *bundle) noexcept {
  return ::rust::Box<::orchard_bundle::Bundle>::from_raw(orchard_bundle$cxxbridge1$from_tx_bundle(bundle));
}

::rust::Vec<::orchard_bundle::Action> Bundle::actions() const noexcept {
  ::rust::MaybeUninit<::rust::Vec<::orchard_bundle::Action>> return$;
  orchard_bundle$cxxbridge1$Bundle$actions(*this, &return$.value);
  return ::std::move(return$.value);
}

::std::size_t Bundle::num_actions() const noexcept {
  return orchard_bundle$cxxbridge1$Bundle$num_actions(*this);
}

bool Bundle::enable_spends() const noexcept {
  return orchard_bundle$cxxbridge1$Bundle$enable_spends(*this);
}

bool Bundle::enable_outputs() const noexcept {
  return orchard_bundle$cxxbridge1$Bundle$enable_outputs(*this);
}

::std::int64_t Bundle::value_balance_zat() const noexcept {
  return orchard_bundle$cxxbridge1$Bundle$value_balance_zat(*this);
}

::std::array<::std::uint8_t, 32> Bundle::anchor() const noexcept {
  ::rust::MaybeUninit<::std::array<::std::uint8_t, 32>> return$;
  orchard_bundle$cxxbridge1$Bundle$anchor(*this, &return$.value);
  return ::std::move(return$.value);
}

::rust::Vec<::std::uint8_t> Bundle::proof() const noexcept {
  ::rust::MaybeUninit<::rust::Vec<::std::uint8_t>> return$;
  orchard_bundle$cxxbridge1$Bundle$proof(*this, &return$.value);
  return ::std::move(return$.value);
}

::std::array<::std::uint8_t, 64> Bundle::binding_sig() const noexcept {
  ::rust::MaybeUninit<::std::array<::std::uint8_t, 64>> return$;
  orchard_bundle$cxxbridge1$Bundle$binding_sig(*this, &return$.value);
  return ::std::move(return$.value);
}
} // namespace orchard_bundle

extern "C" {
::orchard_bundle::Bundle *cxxbridge1$box$orchard_bundle$Bundle$alloc() noexcept;
void cxxbridge1$box$orchard_bundle$Bundle$dealloc(::orchard_bundle::Bundle *) noexcept;
void cxxbridge1$box$orchard_bundle$Bundle$drop(::rust::Box<::orchard_bundle::Bundle> *ptr) noexcept;

void cxxbridge1$rust_vec$orchard_bundle$Action$new(::rust::Vec<::orchard_bundle::Action> const *ptr) noexcept;
void cxxbridge1$rust_vec$orchard_bundle$Action$drop(::rust::Vec<::orchard_bundle::Action> *ptr) noexcept;
::std::size_t cxxbridge1$rust_vec$orchard_bundle$Action$len(::rust::Vec<::orchard_bundle::Action> const *ptr) noexcept;
::std::size_t cxxbridge1$rust_vec$orchard_bundle$Action$capacity(::rust::Vec<::orchard_bundle::Action> const *ptr) noexcept;
::orchard_bundle::Action const *cxxbridge1$rust_vec$orchard_bundle$Action$data(::rust::Vec<::orchard_bundle::Action> const *ptr) noexcept;
void cxxbridge1$rust_vec$orchard_bundle$Action$reserve_total(::rust::Vec<::orchard_bundle::Action> *ptr, ::std::size_t new_cap) noexcept;
void cxxbridge1$rust_vec$orchard_bundle$Action$set_len(::rust::Vec<::orchard_bundle::Action> *ptr, ::std::size_t len) noexcept;
void cxxbridge1$rust_vec$orchard_bundle$Action$truncate(::rust::Vec<::orchard_bundle::Action> *ptr, ::std::size_t len) noexcept;
} // extern "C"

namespace rust {
inline namespace cxxbridge1 {
template <>
::orchard_bundle::Bundle *Box<::orchard_bundle::Bundle>::allocation::alloc() noexcept {
  return cxxbridge1$box$orchard_bundle$Bundle$alloc();
}
template <>
void Box<::orchard_bundle::Bundle>::allocation::dealloc(::orchard_bundle::Bundle *ptr) noexcept {
  cxxbridge1$box$orchard_bundle$Bundle$dealloc(ptr);
}
template <>
void Box<::orchard_bundle::Bundle>::drop() noexcept {
  cxxbridge1$box$orchard_bundle$Bundle$drop(this);
}
template <>
Vec<::orchard_bundle::Action>::Vec() noexcept {
  cxxbridge1$rust_vec$orchard_bundle$Action$new(this);
}
template <>
void Vec<::orchard_bundle::Action>::drop() noexcept {
  return cxxbridge1$rust_vec$orchard_bundle$Action$drop(this);
}
template <>
::std::size_t Vec<::orchard_bundle::Action>::size() const noexcept {
  return cxxbridge1$rust_vec$orchard_bundle$Action$len(this);
}
template <>
::std::size_t Vec<::orchard_bundle::Action>::capacity() const noexcept {
  return cxxbridge1$rust_vec$orchard_bundle$Action$capacity(this);
}
template <>
::orchard_bundle::Action const *Vec<::orchard_bundle::Action>::data() const noexcept {
  return cxxbridge1$rust_vec$orchard_bundle$Action$data(this);
}
template <>
void Vec<::orchard_bundle::Action>::reserve_total(::std::size_t new_cap) noexcept {
  return cxxbridge1$rust_vec$orchard_bundle$Action$reserve_total(this, new_cap);
}
template <>
void Vec<::orchard_bundle::Action>::set_len(::std::size_t len) noexcept {
  return cxxbridge1$rust_vec$orchard_bundle$Action$set_len(this, len);
}
template <>
void Vec<::orchard_bundle::Action>::truncate(::std::size_t len) {
  return cxxbridge1$rust_vec$orchard_bundle$Action$truncate(this, len);
}
} // namespace cxxbridge1
} // namespace rust
