#include "rust/cxx.h"

namespace rust {
inline namespace cxxbridge1 {
template <typename T>
union ManuallyDrop {
  T value;
  ManuallyDrop(T &&value) : value(::std::move(value)) {}
  ~ManuallyDrop() {}
};
} // namespace cxxbridge1
} // namespace rust

namespace sapling {
  struct Bundle;
  struct BundleAssembler;
  struct Prover;
  struct Verifier;
  struct BatchValidator;
}

namespace sapling {
#ifndef CXXBRIDGE1_STRUCT_sapling$Bundle
#define CXXBRIDGE1_STRUCT_sapling$Bundle
struct Bundle final : public ::rust::Opaque {
  ~Bundle() = delete;

private:
  friend ::rust::layout;
  struct layout {
    static ::std::size_t size() noexcept;
    static ::std::size_t align() noexcept;
  };
};
#endif // CXXBRIDGE1_STRUCT_sapling$Bundle

#ifndef CXXBRIDGE1_STRUCT_sapling$BundleAssembler
#define CXXBRIDGE1_STRUCT_sapling$BundleAssembler
struct BundleAssembler final : public ::rust::Opaque {
  bool add_spend(::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &anchor, ::std::array<::std::uint8_t, 32> nullifier, ::std::array<::std::uint8_t, 32> const &rk, ::std::array<::std::uint8_t, 192> zkproof, ::std::array<::std::uint8_t, 64> const &spend_auth_sig) noexcept;
  bool add_output(::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &cmu, ::std::array<::std::uint8_t, 32> ephemeral_key, ::std::array<::std::uint8_t, 580> enc_ciphertext, ::std::array<::std::uint8_t, 80> out_ciphertext, ::std::array<::std::uint8_t, 192> zkproof) noexcept;
  ~BundleAssembler() = delete;

private:
  friend ::rust::layout;
  struct layout {
    static ::std::size_t size() noexcept;
    static ::std::size_t align() noexcept;
  };
};
#endif // CXXBRIDGE1_STRUCT_sapling$BundleAssembler

#ifndef CXXBRIDGE1_STRUCT_sapling$Prover
#define CXXBRIDGE1_STRUCT_sapling$Prover
struct Prover final : public ::rust::Opaque {
  bool create_spend_proof(::std::array<::std::uint8_t, 32> const &ak, ::std::array<::std::uint8_t, 32> const &nsk, ::std::array<::std::uint8_t, 11> const &diversifier, ::std::array<::std::uint8_t, 32> const &rcm, ::std::array<::std::uint8_t, 32> const &ar, ::std::uint64_t value, ::std::array<::std::uint8_t, 32> const &anchor, ::std::array<::std::uint8_t, 1065> const &merkle_path, ::std::array<::std::uint8_t, 32> &cv, ::std::array<::std::uint8_t, 32> &rk_out, ::std::array<::std::uint8_t, 192> &zkproof) noexcept;
  bool create_output_proof(::std::array<::std::uint8_t, 32> const &esk, ::std::array<::std::uint8_t, 43> const &payment_address, ::std::array<::std::uint8_t, 32> const &rcm, ::std::uint64_t value, ::std::array<::std::uint8_t, 32> &cv, ::std::array<::std::uint8_t, 192> &zkproof) noexcept;
  bool binding_sig(::std::int64_t value_balance, ::std::array<::std::uint8_t, 32> const &sighash, ::std::array<::std::uint8_t, 64> &result) noexcept;
  ~Prover() = delete;

private:
  friend ::rust::layout;
  struct layout {
    static ::std::size_t size() noexcept;
    static ::std::size_t align() noexcept;
  };
};
#endif // CXXBRIDGE1_STRUCT_sapling$Prover

#ifndef CXXBRIDGE1_STRUCT_sapling$Verifier
#define CXXBRIDGE1_STRUCT_sapling$Verifier
struct Verifier final : public ::rust::Opaque {
  bool check_spend(::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &anchor, ::std::array<::std::uint8_t, 32> const &nullifier, ::std::array<::std::uint8_t, 32> const &rk, ::std::array<::std::uint8_t, 192> const &zkproof, ::std::array<::std::uint8_t, 64> const &spend_auth_sig, ::std::array<::std::uint8_t, 32> const &sighash_value) noexcept;
  bool check_output(::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &cm, ::std::array<::std::uint8_t, 32> const &ephemeral_key, ::std::array<::std::uint8_t, 192> const &zkproof) noexcept;
  bool final_check(::std::int64_t value_balance, ::std::array<::std::uint8_t, 64> const &binding_sig, ::std::array<::std::uint8_t, 32> const &sighash_value) const noexcept;
  ~Verifier() = delete;

private:
  friend ::rust::layout;
  struct layout {
    static ::std::size_t size() noexcept;
    static ::std::size_t align() noexcept;
  };
};
#endif // CXXBRIDGE1_STRUCT_sapling$Verifier

#ifndef CXXBRIDGE1_STRUCT_sapling$BatchValidator
#define CXXBRIDGE1_STRUCT_sapling$BatchValidator
struct BatchValidator final : public ::rust::Opaque {
  bool check_bundle(::rust::Box<::sapling::Bundle> bundle, ::std::array<::std::uint8_t, 32> sighash) noexcept;
  bool validate() noexcept;
  ~BatchValidator() = delete;

private:
  friend ::rust::layout;
  struct layout {
    static ::std::size_t size() noexcept;
    static ::std::size_t align() noexcept;
  };
};
#endif // CXXBRIDGE1_STRUCT_sapling$BatchValidator

extern "C" {
::std::size_t sapling$cxxbridge1$Bundle$operator$sizeof() noexcept;
::std::size_t sapling$cxxbridge1$Bundle$operator$alignof() noexcept;
::std::size_t sapling$cxxbridge1$BundleAssembler$operator$sizeof() noexcept;
::std::size_t sapling$cxxbridge1$BundleAssembler$operator$alignof() noexcept;

::sapling::BundleAssembler *sapling$cxxbridge1$new_bundle_assembler() noexcept;

bool sapling$cxxbridge1$BundleAssembler$add_spend(::sapling::BundleAssembler &self, ::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &anchor, ::std::array<::std::uint8_t, 32> *nullifier, ::std::array<::std::uint8_t, 32> const &rk, ::std::array<::std::uint8_t, 192> *zkproof, ::std::array<::std::uint8_t, 64> const &spend_auth_sig) noexcept;

bool sapling$cxxbridge1$BundleAssembler$add_output(::sapling::BundleAssembler &self, ::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &cmu, ::std::array<::std::uint8_t, 32> *ephemeral_key, ::std::array<::std::uint8_t, 580> *enc_ciphertext, ::std::array<::std::uint8_t, 80> *out_ciphertext, ::std::array<::std::uint8_t, 192> *zkproof) noexcept;

::sapling::Bundle *sapling$cxxbridge1$finish_bundle_assembly(::sapling::BundleAssembler *assembler, ::std::int64_t value_balance, ::std::array<::std::uint8_t, 64> *binding_sig) noexcept;
::std::size_t sapling$cxxbridge1$Prover$operator$sizeof() noexcept;
::std::size_t sapling$cxxbridge1$Prover$operator$alignof() noexcept;

::sapling::Prover *sapling$cxxbridge1$init_prover() noexcept;

bool sapling$cxxbridge1$Prover$create_spend_proof(::sapling::Prover &self, ::std::array<::std::uint8_t, 32> const &ak, ::std::array<::std::uint8_t, 32> const &nsk, ::std::array<::std::uint8_t, 11> const &diversifier, ::std::array<::std::uint8_t, 32> const &rcm, ::std::array<::std::uint8_t, 32> const &ar, ::std::uint64_t value, ::std::array<::std::uint8_t, 32> const &anchor, ::std::array<::std::uint8_t, 1065> const &merkle_path, ::std::array<::std::uint8_t, 32> &cv, ::std::array<::std::uint8_t, 32> &rk_out, ::std::array<::std::uint8_t, 192> &zkproof) noexcept;

bool sapling$cxxbridge1$Prover$create_output_proof(::sapling::Prover &self, ::std::array<::std::uint8_t, 32> const &esk, ::std::array<::std::uint8_t, 43> const &payment_address, ::std::array<::std::uint8_t, 32> const &rcm, ::std::uint64_t value, ::std::array<::std::uint8_t, 32> &cv, ::std::array<::std::uint8_t, 192> &zkproof) noexcept;

bool sapling$cxxbridge1$Prover$binding_sig(::sapling::Prover &self, ::std::int64_t value_balance, ::std::array<::std::uint8_t, 32> const &sighash, ::std::array<::std::uint8_t, 64> &result) noexcept;
::std::size_t sapling$cxxbridge1$Verifier$operator$sizeof() noexcept;
::std::size_t sapling$cxxbridge1$Verifier$operator$alignof() noexcept;

::sapling::Verifier *sapling$cxxbridge1$init_verifier() noexcept;

bool sapling$cxxbridge1$Verifier$check_spend(::sapling::Verifier &self, ::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &anchor, ::std::array<::std::uint8_t, 32> const &nullifier, ::std::array<::std::uint8_t, 32> const &rk, ::std::array<::std::uint8_t, 192> const &zkproof, ::std::array<::std::uint8_t, 64> const &spend_auth_sig, ::std::array<::std::uint8_t, 32> const &sighash_value) noexcept;

bool sapling$cxxbridge1$Verifier$check_output(::sapling::Verifier &self, ::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &cm, ::std::array<::std::uint8_t, 32> const &ephemeral_key, ::std::array<::std::uint8_t, 192> const &zkproof) noexcept;

bool sapling$cxxbridge1$Verifier$final_check(::sapling::Verifier const &self, ::std::int64_t value_balance, ::std::array<::std::uint8_t, 64> const &binding_sig, ::std::array<::std::uint8_t, 32> const &sighash_value) noexcept;
::std::size_t sapling$cxxbridge1$BatchValidator$operator$sizeof() noexcept;
::std::size_t sapling$cxxbridge1$BatchValidator$operator$alignof() noexcept;

::sapling::BatchValidator *sapling$cxxbridge1$init_batch_validator(bool cache_store) noexcept;

bool sapling$cxxbridge1$BatchValidator$check_bundle(::sapling::BatchValidator &self, ::sapling::Bundle *bundle, ::std::array<::std::uint8_t, 32> *sighash) noexcept;

bool sapling$cxxbridge1$BatchValidator$validate(::sapling::BatchValidator &self) noexcept;
} // extern "C"

::std::size_t Bundle::layout::size() noexcept {
  return sapling$cxxbridge1$Bundle$operator$sizeof();
}

::std::size_t Bundle::layout::align() noexcept {
  return sapling$cxxbridge1$Bundle$operator$alignof();
}

::std::size_t BundleAssembler::layout::size() noexcept {
  return sapling$cxxbridge1$BundleAssembler$operator$sizeof();
}

::std::size_t BundleAssembler::layout::align() noexcept {
  return sapling$cxxbridge1$BundleAssembler$operator$alignof();
}

::rust::Box<::sapling::BundleAssembler> new_bundle_assembler() noexcept {
  return ::rust::Box<::sapling::BundleAssembler>::from_raw(sapling$cxxbridge1$new_bundle_assembler());
}

bool BundleAssembler::add_spend(::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &anchor, ::std::array<::std::uint8_t, 32> nullifier, ::std::array<::std::uint8_t, 32> const &rk, ::std::array<::std::uint8_t, 192> zkproof, ::std::array<::std::uint8_t, 64> const &spend_auth_sig) noexcept {
  ::rust::ManuallyDrop<::std::array<::std::uint8_t, 32>> nullifier$(::std::move(nullifier));
  ::rust::ManuallyDrop<::std::array<::std::uint8_t, 192>> zkproof$(::std::move(zkproof));
  return sapling$cxxbridge1$BundleAssembler$add_spend(*this, cv, anchor, &nullifier$.value, rk, &zkproof$.value, spend_auth_sig);
}

bool BundleAssembler::add_output(::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &cmu, ::std::array<::std::uint8_t, 32> ephemeral_key, ::std::array<::std::uint8_t, 580> enc_ciphertext, ::std::array<::std::uint8_t, 80> out_ciphertext, ::std::array<::std::uint8_t, 192> zkproof) noexcept {
  ::rust::ManuallyDrop<::std::array<::std::uint8_t, 32>> ephemeral_key$(::std::move(ephemeral_key));
  ::rust::ManuallyDrop<::std::array<::std::uint8_t, 580>> enc_ciphertext$(::std::move(enc_ciphertext));
  ::rust::ManuallyDrop<::std::array<::std::uint8_t, 80>> out_ciphertext$(::std::move(out_ciphertext));
  ::rust::ManuallyDrop<::std::array<::std::uint8_t, 192>> zkproof$(::std::move(zkproof));
  return sapling$cxxbridge1$BundleAssembler$add_output(*this, cv, cmu, &ephemeral_key$.value, &enc_ciphertext$.value, &out_ciphertext$.value, &zkproof$.value);
}

::rust::Box<::sapling::Bundle> finish_bundle_assembly(::rust::Box<::sapling::BundleAssembler> assembler, ::std::int64_t value_balance, ::std::array<::std::uint8_t, 64> binding_sig) noexcept {
  ::rust::ManuallyDrop<::std::array<::std::uint8_t, 64>> binding_sig$(::std::move(binding_sig));
  return ::rust::Box<::sapling::Bundle>::from_raw(sapling$cxxbridge1$finish_bundle_assembly(assembler.into_raw(), value_balance, &binding_sig$.value));
}

::std::size_t Prover::layout::size() noexcept {
  return sapling$cxxbridge1$Prover$operator$sizeof();
}

::std::size_t Prover::layout::align() noexcept {
  return sapling$cxxbridge1$Prover$operator$alignof();
}

::rust::Box<::sapling::Prover> init_prover() noexcept {
  return ::rust::Box<::sapling::Prover>::from_raw(sapling$cxxbridge1$init_prover());
}

bool Prover::create_spend_proof(::std::array<::std::uint8_t, 32> const &ak, ::std::array<::std::uint8_t, 32> const &nsk, ::std::array<::std::uint8_t, 11> const &diversifier, ::std::array<::std::uint8_t, 32> const &rcm, ::std::array<::std::uint8_t, 32> const &ar, ::std::uint64_t value, ::std::array<::std::uint8_t, 32> const &anchor, ::std::array<::std::uint8_t, 1065> const &merkle_path, ::std::array<::std::uint8_t, 32> &cv, ::std::array<::std::uint8_t, 32> &rk_out, ::std::array<::std::uint8_t, 192> &zkproof) noexcept {
  return sapling$cxxbridge1$Prover$create_spend_proof(*this, ak, nsk, diversifier, rcm, ar, value, anchor, merkle_path, cv, rk_out, zkproof);
}

bool Prover::create_output_proof(::std::array<::std::uint8_t, 32> const &esk, ::std::array<::std::uint8_t, 43> const &payment_address, ::std::array<::std::uint8_t, 32> const &rcm, ::std::uint64_t value, ::std::array<::std::uint8_t, 32> &cv, ::std::array<::std::uint8_t, 192> &zkproof) noexcept {
  return sapling$cxxbridge1$Prover$create_output_proof(*this, esk, payment_address, rcm, value, cv, zkproof);
}

bool Prover::binding_sig(::std::int64_t value_balance, ::std::array<::std::uint8_t, 32> const &sighash, ::std::array<::std::uint8_t, 64> &result) noexcept {
  return sapling$cxxbridge1$Prover$binding_sig(*this, value_balance, sighash, result);
}

::std::size_t Verifier::layout::size() noexcept {
  return sapling$cxxbridge1$Verifier$operator$sizeof();
}

::std::size_t Verifier::layout::align() noexcept {
  return sapling$cxxbridge1$Verifier$operator$alignof();
}

::rust::Box<::sapling::Verifier> init_verifier() noexcept {
  return ::rust::Box<::sapling::Verifier>::from_raw(sapling$cxxbridge1$init_verifier());
}

bool Verifier::check_spend(::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &anchor, ::std::array<::std::uint8_t, 32> const &nullifier, ::std::array<::std::uint8_t, 32> const &rk, ::std::array<::std::uint8_t, 192> const &zkproof, ::std::array<::std::uint8_t, 64> const &spend_auth_sig, ::std::array<::std::uint8_t, 32> const &sighash_value) noexcept {
  return sapling$cxxbridge1$Verifier$check_spend(*this, cv, anchor, nullifier, rk, zkproof, spend_auth_sig, sighash_value);
}

bool Verifier::check_output(::std::array<::std::uint8_t, 32> const &cv, ::std::array<::std::uint8_t, 32> const &cm, ::std::array<::std::uint8_t, 32> const &ephemeral_key, ::std::array<::std::uint8_t, 192> const &zkproof) noexcept {
  return sapling$cxxbridge1$Verifier$check_output(*this, cv, cm, ephemeral_key, zkproof);
}

bool Verifier::final_check(::std::int64_t value_balance, ::std::array<::std::uint8_t, 64> const &binding_sig, ::std::array<::std::uint8_t, 32> const &sighash_value) const noexcept {
  return sapling$cxxbridge1$Verifier$final_check(*this, value_balance, binding_sig, sighash_value);
}

::std::size_t BatchValidator::layout::size() noexcept {
  return sapling$cxxbridge1$BatchValidator$operator$sizeof();
}

::std::size_t BatchValidator::layout::align() noexcept {
  return sapling$cxxbridge1$BatchValidator$operator$alignof();
}

::rust::Box<::sapling::BatchValidator> init_batch_validator(bool cache_store) noexcept {
  return ::rust::Box<::sapling::BatchValidator>::from_raw(sapling$cxxbridge1$init_batch_validator(cache_store));
}

bool BatchValidator::check_bundle(::rust::Box<::sapling::Bundle> bundle, ::std::array<::std::uint8_t, 32> sighash) noexcept {
  ::rust::ManuallyDrop<::std::array<::std::uint8_t, 32>> sighash$(::std::move(sighash));
  return sapling$cxxbridge1$BatchValidator$check_bundle(*this, bundle.into_raw(), &sighash$.value);
}

bool BatchValidator::validate() noexcept {
  return sapling$cxxbridge1$BatchValidator$validate(*this);
}
} // namespace sapling

extern "C" {
::sapling::BundleAssembler *cxxbridge1$box$sapling$BundleAssembler$alloc() noexcept;
void cxxbridge1$box$sapling$BundleAssembler$dealloc(::sapling::BundleAssembler *) noexcept;
void cxxbridge1$box$sapling$BundleAssembler$drop(::rust::Box<::sapling::BundleAssembler> *ptr) noexcept;

::sapling::Bundle *cxxbridge1$box$sapling$Bundle$alloc() noexcept;
void cxxbridge1$box$sapling$Bundle$dealloc(::sapling::Bundle *) noexcept;
void cxxbridge1$box$sapling$Bundle$drop(::rust::Box<::sapling::Bundle> *ptr) noexcept;

::sapling::Prover *cxxbridge1$box$sapling$Prover$alloc() noexcept;
void cxxbridge1$box$sapling$Prover$dealloc(::sapling::Prover *) noexcept;
void cxxbridge1$box$sapling$Prover$drop(::rust::Box<::sapling::Prover> *ptr) noexcept;

::sapling::Verifier *cxxbridge1$box$sapling$Verifier$alloc() noexcept;
void cxxbridge1$box$sapling$Verifier$dealloc(::sapling::Verifier *) noexcept;
void cxxbridge1$box$sapling$Verifier$drop(::rust::Box<::sapling::Verifier> *ptr) noexcept;

::sapling::BatchValidator *cxxbridge1$box$sapling$BatchValidator$alloc() noexcept;
void cxxbridge1$box$sapling$BatchValidator$dealloc(::sapling::BatchValidator *) noexcept;
void cxxbridge1$box$sapling$BatchValidator$drop(::rust::Box<::sapling::BatchValidator> *ptr) noexcept;
} // extern "C"

namespace rust {
inline namespace cxxbridge1 {
template <>
::sapling::BundleAssembler *Box<::sapling::BundleAssembler>::allocation::alloc() noexcept {
  return cxxbridge1$box$sapling$BundleAssembler$alloc();
}
template <>
void Box<::sapling::BundleAssembler>::allocation::dealloc(::sapling::BundleAssembler *ptr) noexcept {
  cxxbridge1$box$sapling$BundleAssembler$dealloc(ptr);
}
template <>
void Box<::sapling::BundleAssembler>::drop() noexcept {
  cxxbridge1$box$sapling$BundleAssembler$drop(this);
}
template <>
::sapling::Bundle *Box<::sapling::Bundle>::allocation::alloc() noexcept {
  return cxxbridge1$box$sapling$Bundle$alloc();
}
template <>
void Box<::sapling::Bundle>::allocation::dealloc(::sapling::Bundle *ptr) noexcept {
  cxxbridge1$box$sapling$Bundle$dealloc(ptr);
}
template <>
void Box<::sapling::Bundle>::drop() noexcept {
  cxxbridge1$box$sapling$Bundle$drop(this);
}
template <>
::sapling::Prover *Box<::sapling::Prover>::allocation::alloc() noexcept {
  return cxxbridge1$box$sapling$Prover$alloc();
}
template <>
void Box<::sapling::Prover>::allocation::dealloc(::sapling::Prover *ptr) noexcept {
  cxxbridge1$box$sapling$Prover$dealloc(ptr);
}
template <>
void Box<::sapling::Prover>::drop() noexcept {
  cxxbridge1$box$sapling$Prover$drop(this);
}
template <>
::sapling::Verifier *Box<::sapling::Verifier>::allocation::alloc() noexcept {
  return cxxbridge1$box$sapling$Verifier$alloc();
}
template <>
void Box<::sapling::Verifier>::allocation::dealloc(::sapling::Verifier *ptr) noexcept {
  cxxbridge1$box$sapling$Verifier$dealloc(ptr);
}
template <>
void Box<::sapling::Verifier>::drop() noexcept {
  cxxbridge1$box$sapling$Verifier$drop(this);
}
template <>
::sapling::BatchValidator *Box<::sapling::BatchValidator>::allocation::alloc() noexcept {
  return cxxbridge1$box$sapling$BatchValidator$alloc();
}
template <>
void Box<::sapling::BatchValidator>::allocation::dealloc(::sapling::BatchValidator *ptr) noexcept {
  cxxbridge1$box$sapling$BatchValidator$dealloc(ptr);
}
template <>
void Box<::sapling::BatchValidator>::drop() noexcept {
  cxxbridge1$box$sapling$BatchValidator$drop(this);
}
} // namespace cxxbridge1
} // namespace rust
