#include "rust/cxx.h"

namespace rust {
inline namespace cxxbridge1 {
namespace repr {
struct PtrLen final {
  void *ptr;
  ::std::size_t len;
};
} // namespace repr

namespace detail {
template <typename T, typename = void *>
struct operator_new {
  void *operator()(::std::size_t sz) { return ::operator new(sz); }
};

template <typename T>
struct operator_new<T, decltype(T::operator new(sizeof(T)))> {
  void *operator()(::std::size_t sz) { return T::operator new(sz); }
};
} // namespace detail

template <typename T>
union ManuallyDrop {
  T value;
  ManuallyDrop(T &&value) : value(::std::move(value)) {}
  ~ManuallyDrop() {}
};

template <typename T>
union MaybeUninit {
  T value;
  void *operator new(::std::size_t sz) { return detail::operator_new<T>{}(sz); }
  MaybeUninit() {}
  ~MaybeUninit() {}
};

namespace {
template <>
class impl<Error> final {
public:
  static Error error(repr::PtrLen repr) noexcept {
    Error error;
    error.msg = static_cast<char const *>(repr.ptr);
    error.len = repr.len;
    return error;
  }
};
} // namespace
} // namespace cxxbridge1
} // namespace rust

namespace wallet {
  struct SaplingDecryptionResult;
  struct Network;
  struct BatchScanner;
  struct BatchResult;
}

namespace wallet {
#ifndef CXXBRIDGE1_STRUCT_wallet$SaplingDecryptionResult
#define CXXBRIDGE1_STRUCT_wallet$SaplingDecryptionResult
struct SaplingDecryptionResult final {
  ::std::array<::std::uint8_t, 32> txid;
  ::std::uint32_t output;
  ::std::array<::std::uint8_t, 32> ivk;
  ::std::array<::std::uint8_t, 11> diversifier;
  ::std::array<::std::uint8_t, 32> pk_d;

  using IsRelocatable = ::std::true_type;
};
#endif // CXXBRIDGE1_STRUCT_wallet$SaplingDecryptionResult

#ifndef CXXBRIDGE1_STRUCT_wallet$Network
#define CXXBRIDGE1_STRUCT_wallet$Network
struct Network final : public ::rust::Opaque {
  ~Network() = delete;

private:
  friend ::rust::layout;
  struct layout {
    static ::std::size_t size() noexcept;
    static ::std::size_t align() noexcept;
  };
};
#endif // CXXBRIDGE1_STRUCT_wallet$Network

#ifndef CXXBRIDGE1_STRUCT_wallet$BatchScanner
#define CXXBRIDGE1_STRUCT_wallet$BatchScanner
struct BatchScanner final : public ::rust::Opaque {
  ::std::size_t get_dynamic_usage() const noexcept;
  void add_transaction(::std::array<::std::uint8_t, 32> block_tag, ::rust::Slice<::std::uint8_t const> tx_bytes, ::std::uint32_t height);
  void flush() noexcept;
  ::rust::Box<::wallet::BatchResult> collect_results(::std::array<::std::uint8_t, 32> block_tag, ::std::array<::std::uint8_t, 32> txid) noexcept;
  ~BatchScanner() = delete;

private:
  friend ::rust::layout;
  struct layout {
    static ::std::size_t size() noexcept;
    static ::std::size_t align() noexcept;
  };
};
#endif // CXXBRIDGE1_STRUCT_wallet$BatchScanner

#ifndef CXXBRIDGE1_STRUCT_wallet$BatchResult
#define CXXBRIDGE1_STRUCT_wallet$BatchResult
struct BatchResult final : public ::rust::Opaque {
  ::rust::Vec<::wallet::SaplingDecryptionResult> get_sapling() const noexcept;
  ~BatchResult() = delete;

private:
  friend ::rust::layout;
  struct layout {
    static ::std::size_t size() noexcept;
    static ::std::size_t align() noexcept;
  };
};
#endif // CXXBRIDGE1_STRUCT_wallet$BatchResult

extern "C" {
::std::size_t wallet$cxxbridge1$Network$operator$sizeof() noexcept;
::std::size_t wallet$cxxbridge1$Network$operator$alignof() noexcept;
::std::size_t wallet$cxxbridge1$BatchScanner$operator$sizeof() noexcept;
::std::size_t wallet$cxxbridge1$BatchScanner$operator$alignof() noexcept;
::std::size_t wallet$cxxbridge1$BatchResult$operator$sizeof() noexcept;
::std::size_t wallet$cxxbridge1$BatchResult$operator$alignof() noexcept;

::rust::repr::PtrLen wallet$cxxbridge1$network(::rust::Str network, ::std::int32_t overwinter, ::std::int32_t sapling, ::std::int32_t blossom, ::std::int32_t heartwood, ::std::int32_t canopy, ::std::int32_t nu5, ::rust::Box<::wallet::Network> *return$) noexcept;

::rust::repr::PtrLen wallet$cxxbridge1$init_batch_scanner(::wallet::Network const &network, ::rust::Slice<::std::array<::std::uint8_t, 32> const> sapling_ivks, ::rust::Box<::wallet::BatchScanner> *return$) noexcept;

::std::size_t wallet$cxxbridge1$BatchScanner$get_dynamic_usage(::wallet::BatchScanner const &self) noexcept;

::rust::repr::PtrLen wallet$cxxbridge1$BatchScanner$add_transaction(::wallet::BatchScanner &self, ::std::array<::std::uint8_t, 32> *block_tag, ::rust::Slice<::std::uint8_t const> tx_bytes, ::std::uint32_t height) noexcept;

void wallet$cxxbridge1$BatchScanner$flush(::wallet::BatchScanner &self) noexcept;

::wallet::BatchResult *wallet$cxxbridge1$BatchScanner$collect_results(::wallet::BatchScanner &self, ::std::array<::std::uint8_t, 32> *block_tag, ::std::array<::std::uint8_t, 32> *txid) noexcept;

void wallet$cxxbridge1$BatchResult$get_sapling(::wallet::BatchResult const &self, ::rust::Vec<::wallet::SaplingDecryptionResult> *return$) noexcept;
} // extern "C"

::std::size_t Network::layout::size() noexcept {
  return wallet$cxxbridge1$Network$operator$sizeof();
}

::std::size_t Network::layout::align() noexcept {
  return wallet$cxxbridge1$Network$operator$alignof();
}

::std::size_t BatchScanner::layout::size() noexcept {
  return wallet$cxxbridge1$BatchScanner$operator$sizeof();
}

::std::size_t BatchScanner::layout::align() noexcept {
  return wallet$cxxbridge1$BatchScanner$operator$alignof();
}

::std::size_t BatchResult::layout::size() noexcept {
  return wallet$cxxbridge1$BatchResult$operator$sizeof();
}

::std::size_t BatchResult::layout::align() noexcept {
  return wallet$cxxbridge1$BatchResult$operator$alignof();
}

::rust::Box<::wallet::Network> network(::rust::Str network, ::std::int32_t overwinter, ::std::int32_t sapling, ::std::int32_t blossom, ::std::int32_t heartwood, ::std::int32_t canopy, ::std::int32_t nu5) {
  ::rust::MaybeUninit<::rust::Box<::wallet::Network>> return$;
  ::rust::repr::PtrLen error$ = wallet$cxxbridge1$network(network, overwinter, sapling, blossom, heartwood, canopy, nu5, &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

::rust::Box<::wallet::BatchScanner> init_batch_scanner(::wallet::Network const &network, ::rust::Slice<::std::array<::std::uint8_t, 32> const> sapling_ivks) {
  ::rust::MaybeUninit<::rust::Box<::wallet::BatchScanner>> return$;
  ::rust::repr::PtrLen error$ = wallet$cxxbridge1$init_batch_scanner(network, sapling_ivks, &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

::std::size_t BatchScanner::get_dynamic_usage() const noexcept {
  return wallet$cxxbridge1$BatchScanner$get_dynamic_usage(*this);
}

void BatchScanner::add_transaction(::std::array<::std::uint8_t, 32> block_tag, ::rust::Slice<::std::uint8_t const> tx_bytes, ::std::uint32_t height) {
  ::rust::ManuallyDrop<::std::array<::std::uint8_t, 32>> block_tag$(::std::move(block_tag));
  ::rust::repr::PtrLen error$ = wallet$cxxbridge1$BatchScanner$add_transaction(*this, &block_tag$.value, tx_bytes, height);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
}

void BatchScanner::flush() noexcept {
  wallet$cxxbridge1$BatchScanner$flush(*this);
}

::rust::Box<::wallet::BatchResult> BatchScanner::collect_results(::std::array<::std::uint8_t, 32> block_tag, ::std::array<::std::uint8_t, 32> txid) noexcept {
  ::rust::ManuallyDrop<::std::array<::std::uint8_t, 32>> block_tag$(::std::move(block_tag));
  ::rust::ManuallyDrop<::std::array<::std::uint8_t, 32>> txid$(::std::move(txid));
  return ::rust::Box<::wallet::BatchResult>::from_raw(wallet$cxxbridge1$BatchScanner$collect_results(*this, &block_tag$.value, &txid$.value));
}

::rust::Vec<::wallet::SaplingDecryptionResult> BatchResult::get_sapling() const noexcept {
  ::rust::MaybeUninit<::rust::Vec<::wallet::SaplingDecryptionResult>> return$;
  wallet$cxxbridge1$BatchResult$get_sapling(*this, &return$.value);
  return ::std::move(return$.value);
}
} // namespace wallet

extern "C" {
::wallet::Network *cxxbridge1$box$wallet$Network$alloc() noexcept;
void cxxbridge1$box$wallet$Network$dealloc(::wallet::Network *) noexcept;
void cxxbridge1$box$wallet$Network$drop(::rust::Box<::wallet::Network> *ptr) noexcept;

::wallet::BatchScanner *cxxbridge1$box$wallet$BatchScanner$alloc() noexcept;
void cxxbridge1$box$wallet$BatchScanner$dealloc(::wallet::BatchScanner *) noexcept;
void cxxbridge1$box$wallet$BatchScanner$drop(::rust::Box<::wallet::BatchScanner> *ptr) noexcept;

::wallet::BatchResult *cxxbridge1$box$wallet$BatchResult$alloc() noexcept;
void cxxbridge1$box$wallet$BatchResult$dealloc(::wallet::BatchResult *) noexcept;
void cxxbridge1$box$wallet$BatchResult$drop(::rust::Box<::wallet::BatchResult> *ptr) noexcept;

void cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$new(::rust::Vec<::wallet::SaplingDecryptionResult> const *ptr) noexcept;
void cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$drop(::rust::Vec<::wallet::SaplingDecryptionResult> *ptr) noexcept;
::std::size_t cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$len(::rust::Vec<::wallet::SaplingDecryptionResult> const *ptr) noexcept;
::std::size_t cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$capacity(::rust::Vec<::wallet::SaplingDecryptionResult> const *ptr) noexcept;
::wallet::SaplingDecryptionResult const *cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$data(::rust::Vec<::wallet::SaplingDecryptionResult> const *ptr) noexcept;
void cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$reserve_total(::rust::Vec<::wallet::SaplingDecryptionResult> *ptr, ::std::size_t new_cap) noexcept;
void cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$set_len(::rust::Vec<::wallet::SaplingDecryptionResult> *ptr, ::std::size_t len) noexcept;
void cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$truncate(::rust::Vec<::wallet::SaplingDecryptionResult> *ptr, ::std::size_t len) noexcept;
} // extern "C"

namespace rust {
inline namespace cxxbridge1 {
template <>
::wallet::Network *Box<::wallet::Network>::allocation::alloc() noexcept {
  return cxxbridge1$box$wallet$Network$alloc();
}
template <>
void Box<::wallet::Network>::allocation::dealloc(::wallet::Network *ptr) noexcept {
  cxxbridge1$box$wallet$Network$dealloc(ptr);
}
template <>
void Box<::wallet::Network>::drop() noexcept {
  cxxbridge1$box$wallet$Network$drop(this);
}
template <>
::wallet::BatchScanner *Box<::wallet::BatchScanner>::allocation::alloc() noexcept {
  return cxxbridge1$box$wallet$BatchScanner$alloc();
}
template <>
void Box<::wallet::BatchScanner>::allocation::dealloc(::wallet::BatchScanner *ptr) noexcept {
  cxxbridge1$box$wallet$BatchScanner$dealloc(ptr);
}
template <>
void Box<::wallet::BatchScanner>::drop() noexcept {
  cxxbridge1$box$wallet$BatchScanner$drop(this);
}
template <>
::wallet::BatchResult *Box<::wallet::BatchResult>::allocation::alloc() noexcept {
  return cxxbridge1$box$wallet$BatchResult$alloc();
}
template <>
void Box<::wallet::BatchResult>::allocation::dealloc(::wallet::BatchResult *ptr) noexcept {
  cxxbridge1$box$wallet$BatchResult$dealloc(ptr);
}
template <>
void Box<::wallet::BatchResult>::drop() noexcept {
  cxxbridge1$box$wallet$BatchResult$drop(this);
}
template <>
Vec<::wallet::SaplingDecryptionResult>::Vec() noexcept {
  cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$new(this);
}
template <>
void Vec<::wallet::SaplingDecryptionResult>::drop() noexcept {
  return cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$drop(this);
}
template <>
::std::size_t Vec<::wallet::SaplingDecryptionResult>::size() const noexcept {
  return cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$len(this);
}
template <>
::std::size_t Vec<::wallet::SaplingDecryptionResult>::capacity() const noexcept {
  return cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$capacity(this);
}
template <>
::wallet::SaplingDecryptionResult const *Vec<::wallet::SaplingDecryptionResult>::data() const noexcept {
  return cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$data(this);
}
template <>
void Vec<::wallet::SaplingDecryptionResult>::reserve_total(::std::size_t new_cap) noexcept {
  return cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$reserve_total(this, new_cap);
}
template <>
void Vec<::wallet::SaplingDecryptionResult>::set_len(::std::size_t len) noexcept {
  return cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$set_len(this, len);
}
template <>
void Vec<::wallet::SaplingDecryptionResult>::truncate(::std::size_t len) {
  return cxxbridge1$rust_vec$wallet$SaplingDecryptionResult$truncate(this, len);
}
} // namespace cxxbridge1
} // namespace rust
